@@ -1,5 +1,5 @@
 /* MiniDLNA media server
- * Copyright (C) 2013  NETGEAR
+ * Copyright (C) 2013-2017  NETGEAR
  *
  * This file is part of MiniDLNA.
  *
@@ -37,6 +37,7 @@
 #define FLAG_AUDIO_ONLY         0x00000400
 #define FLAG_FORCE_SORT         0x00000800
 #define FLAG_CAPTION_RES        0x00001000
+#define FLAG_SKIP_DLNA_PN       0x00002000 /* during browsing */
 /* Response-related flags */
 #define FLAG_HAS_CAPTIONS       0x80000000
 #define RESPONSE_FLAGS          0xF0000000
@@ -69,13 +70,16 @@ enum client_types {
 	ESamsungSeriesB,
 	ESamsungSeriesCDEBDP,
 	ESamsungSeriesCDE,
+	ESamsungBDJ5500,
 	ESonyBDP,
 	ESonyBravia,
 	ESonyInternetTV,
 	EToshibaTV,
+	EHyundaiTV,
 	EAsusOPlay,
 	EBubbleUPnP,
 	ENetFrontLivingConnect,
+	EKodi,
 	EStandardDLNA150,
 	EStandardUPnP
 };
